searchData={"items":[{"type":"module","title":"Google.Protobuf","doc":"Utility functions for working with Google Protobuf structs.","ref":"Google.Protobuf.html"},{"type":"function","title":"Google.Protobuf.from_datetime/1","doc":"Converts a `Google.Protobuf.Timestamp` struct to a `DateTime` struct.","ref":"Google.Protobuf.html#from_datetime/1"},{"type":"function","title":"Examples - Google.Protobuf.from_datetime/1","doc":"iex> from_datetime(~U[1970-01-01 00:00:05.000000Z])\n      %Google.Protobuf.Timestamp{seconds: 5, nanos: 0}","ref":"Google.Protobuf.html#from_datetime/1-examples"},{"type":"function","title":"Google.Protobuf.from_map/1","doc":"Converts a `t:map()` to a `Google.Protobuf.Struct` struct recursively\nwrapping values in their `Google.Protobuf.Value` equivalents.","ref":"Google.Protobuf.html#from_map/1"},{"type":"function","title":"Examples - Google.Protobuf.from_map/1","doc":"iex> from_map(%{})\n    %Google.Protobuf.Struct{}","ref":"Google.Protobuf.html#from_map/1-examples"},{"type":"function","title":"Google.Protobuf.to_datetime/1","doc":"Converts a `DateTime` struct to a `Google.Protobuf.Timestamp` struct.\n\nNote: Elixir `DateTime.from_unix!/2` will convert units to\nmicroseconds internally. Nanosecond precision is not guaranteed.\nSee examples for details.","ref":"Google.Protobuf.html#to_datetime/1"},{"type":"function","title":"Examples - Google.Protobuf.to_datetime/1","doc":"iex> to_datetime(%Google.Protobuf.Timestamp{seconds: 5, nanos: 0})\n    ~U[1970-01-01 00:00:05.000000Z]\n\n    iex> one = to_datetime(%Google.Protobuf.Timestamp{seconds: 10, nanos: 100})\n    ...> two = to_datetime(%Google.Protobuf.Timestamp{seconds: 10, nanos: 105})\n    ...> DateTime.diff(one, two, :nanosecond)\n    0","ref":"Google.Protobuf.html#to_datetime/1-examples"},{"type":"function","title":"Google.Protobuf.to_map/1","doc":"Converts a `Google.Protobuf.Struct` struct to a `t:map()` recursively\nconverting values to their Elixir equivalents.","ref":"Google.Protobuf.html#to_map/1"},{"type":"function","title":"Examples - Google.Protobuf.to_map/1","doc":"iex> to_map(%Google.Protobuf.Struct{})\n    %{}\n\n    iex> to_map(%Google.Protobuf.Struct{\n      fields: %{\n        \"key_one\" => %Google.Protobuf.Value{\n          kind: {:string_value, \"value_one\"},\n        },\n        \"key_two\" => %Google.Protobuf.Value{\n          kind: {:number_value, 1234.0},\n        }\n      },\n    })\n    %{\"key_one\" => \"value_one\", \"key_two\" => 1234.0}","ref":"Google.Protobuf.html#to_map/1-examples"},{"type":"behaviour","title":"Protobuf","doc":"`protoc` should always be used to generate code instead of writing the code by hand.","ref":"Protobuf.html"},{"type":"behaviour","title":"Usage - Protobuf","doc":"By `use`'ing this module, macros defined in `Protobuf.DSL` will be injected. Most of these\nmacros are equal to definition in `.proto` schemas.\n\n    defmodule Foo do\n      use Protobuf, syntax: :proto3\n\n      field :a, 1, type: :int32\n      field :b, 2, type: :string\n    end\n\nYour Protobuf message (module) is just a normal Elixir struct. The default values in the\nstruct match the correct ones for the Protobuf schema definition. You can construct\nnew messages by hand:\n\n    foo = %Foo{a: 1}\n    Protobuf.encode(foo)\n    #=> <<...>>\n\nExcept functions in \"Callbacks\", some other functions may be defined:\n\n* Extension functions when your Protobuf message use extensions. See `Protobuf.Extension` for details.\n  * `put_extension(struct, extension_mod, field, value)`\n  * `get_extension(struct, extension_mod, field, default \\ nil)`","ref":"Protobuf.html#module-usage"},{"type":"behaviour","title":"Options - Protobuf","doc":"These are the options that you can pass to `use Protobuf`:\n\n  * `:syntax` - The syntax of the schema. Can be `:proto2` or `:proto3`. Defaults to `:proto2`.\n  * `:enum` - A boolean that tells whether this message is an enum. Defaults to `false`.\n  * `:map` - A boolean that tells whether this message is a map. Defaults to `false`.","ref":"Protobuf.html#module-options"},{"type":"callback","title":"Protobuf.decode/1","doc":"Decodes a Protobuf binary into a struct.\n\nErrors may be raised if there's something wrong in the binary.","ref":"Protobuf.html#c:decode/1"},{"type":"function","title":"Protobuf.decode/2","doc":"Decodes the given binary data interpreting it as the Protobuf message `module`.\n\nThis function raises an error if anything goes wrong with decoding.","ref":"Protobuf.html#decode/2"},{"type":"function","title":"Examples - Protobuf.decode/2","doc":"Protobuf.decode(<<...>>, MyMessage)\n    #=> %MyMessage{...}\n\n    Protobuf.decode(<<\"bad data\">>, MyMessage)\n    #=> ** (Protobuf.DecodeError) ...","ref":"Protobuf.html#decode/2-examples"},{"type":"function","title":"Protobuf.encode/1","doc":"Encodes the given Protobuf struct into a binary.\n\nIf you want to encode to iodata instead, see `encode_to_iodata/1`.","ref":"Protobuf.html#encode/1"},{"type":"function","title":"Examples - Protobuf.encode/1","doc":"Protobuf.encode(%MyMessage{...})\n    #=> <<...>>","ref":"Protobuf.html#encode/1-examples"},{"type":"callback","title":"Protobuf.encode/1","doc":"Encodes the given struct into to a Protobuf binary.\n\nErrors may be raised if there's something wrong in the struct.\n\nIf you want to encode to iodata instead of to a binary, use `encode_to_iodata/1`.","ref":"Protobuf.html#c:encode/1"},{"type":"function","title":"Protobuf.encode_to_iodata/1","doc":"Encodes the given Protobuf struct into iodata.","ref":"Protobuf.html#encode_to_iodata/1"},{"type":"function","title":"Examples - Protobuf.encode_to_iodata/1","doc":"Protobuf.encode_to_iodata(%MyMessage{...})\n    #=> [...]","ref":"Protobuf.html#encode_to_iodata/1-examples"},{"type":"function","title":"Protobuf.get_unknown_fields/1","doc":"Returns the unknown fields that were decoded but were not understood from the schema.\n\nIn Protobuf, you can decode a payload (for the same message) encoded with a different version of\nthe schema for that message. This can result in, for example, the payload containing fields that\ncannot be decoded correctly because they're not present in the schema used for decoding. These\nfields are skipped, but in some cases you might wish to preserve them in order to re-encode\nthem, log them, or other. A common case is having to do \"round-trips\" with messages: you decode\na payload, update the resulting message somehow, and re-encode it for future use. In these\ncases, you would probably want to re-encode the unknown fields to maintain symmetry.\n\nThe returned value of this function is a list of `{field_number, wire_type, field_value}` tuples where\n`field_number` is the number of the unknown field in the schema used for its encoding and\n`field_value` is its value. The library does not make any assumptions on the value of the\nfield since it can't know for sure. This means that, for example, it can't properly decode\nan integer as signed or unsigned. The only guarantee is that the unknown fields are re-encoded\ncorrectly.\n\nThe reason why these fields need to be accessed through this function is that the way they\nare stored in the struct is private.","ref":"Protobuf.html#get_unknown_fields/1"},{"type":"function","title":"Examples - Protobuf.get_unknown_fields/1","doc":"Imagine you have this Protobuf schema:\n\n    message User {\n      string email = 1;\n    }\n\nYou encode this:\n\n    payload = Protobuf.encode(User.new!(email: \"user@example.com\"))\n    #=> <<...>>\n\nNow, you try to decode this payload using this schema instead:\n\n    message User {\n      string full_name = 2;\n    }\n\nIn this case, this function will return the decoded unknown field:\n\n    message = User.decode(<<...>>)\n    Protobuf.get_unknown_fields(message)\n    #=> [{_field_number = 1, _wire_type = 3, \"user@example.com\"}]","ref":"Protobuf.html#get_unknown_fields/1-examples"},{"type":"function","title":"Protobuf.load_extensions/0","doc":"Loads extensions modules.\n\nThis function should be called in your application's `c:Application.start/2` callback,\nas seen in the example below, if you wish to use extensions.","ref":"Protobuf.html#load_extensions/0"},{"type":"function","title":"Example - Protobuf.load_extensions/0","doc":"@impl Application\n    def start(_type, _args) do\n      Protobuf.load_extensions()\n      Supervisor.start_link([], strategy: :one_for_one)\n    end","ref":"Protobuf.html#load_extensions/0-example"},{"type":"callback","title":"Protobuf.new/0","doc":"Builds a blank struct with default values.\n\n> #### Deprecated {: .warning}\n>\n> This is deprecated in favor of building the struct with `%MyMessage{...}` or using\n> `struct/1`.","ref":"Protobuf.html#c:new/0"},{"type":"callback","title":"Protobuf.new/1","doc":"Builds and updates the struct with passed fields.\n\nThis function will:\n\n  * Recursively call `c:new/1` for embedded fields\n  * Create structs using `struct/2` for keyword lists and maps\n  * Create the correct struct if passed the wrong struct\n  * Call `c:new/1` for each element in the list for repeated fields\n\n> #### Deprecated {: .warning}\n> This is deprecated in favor of building the struct with `%MyMessage{...}` or using\n> `struct/2`.","ref":"Protobuf.html#c:new/1"},{"type":"callback","title":"Examples - Protobuf.new/1","doc":"MyMessage.new(field1: \"foo\")\n    #=> %MyMessage{field1: \"foo\", ...}\n\n    MyMessage.new(field1: [field2: \"foo\"])\n    #=> %MyMessage{field1: %MySubMessage{field2: \"foo\"}}\n\n    MyMessage.new(field1: %WrongStruct{field2: \"foo\"})\n    #=> %MyMessage{field1: %MySubMessage{field2: \"foo\"}}\n\n    MyMessage.new(repeated: [%{field1: \"foo\"}, %{field1: \"bar\"}])\n    #=> %MyMessage{repeated: [%MyRepeated{field1: \"foo\"}, %MyRepeated{field1: \"bar\"}]}","ref":"Protobuf.html#c:new/1-examples"},{"type":"callback","title":"Protobuf.new!/1","doc":"Similar to `c:new/1`, but use `struct!/2` to build the struct, so\nerrors will be raised if unknown keys are passed.\n\n> #### Deprecated {: .warning}\n> This is deprecated in favor of building the struct with `%MyMessage{...}` or using\n> `struct!/2` directly.","ref":"Protobuf.html#c:new!/1"},{"type":"callback","title":"Protobuf.transform_module/0","doc":"Returns `nil` or a transformer module that implements the `Protobuf.TransformModule`\nbehaviour.\n\nThis function is overridable in your module.","ref":"Protobuf.html#c:transform_module/0"},{"type":"type","title":"Protobuf.unknown_field/0","doc":"Represents a field that existed in a decoded message, but didn't exist in the\nused protobuf definition.\n\nSee `get_unknown_fields/1` for more information.","ref":"Protobuf.html#t:unknown_field/0"},{"type":"type","title":"Protobuf.wire_type/0","doc":"Internal wire type","ref":"Protobuf.html#t:wire_type/0"},{"type":"module","title":"Protobuf.DSL","doc":"","ref":"Protobuf.DSL.html"},{"type":"macro","title":"Protobuf.DSL.extend/4","doc":"Define \"extend\" for a message(the first argument module).","ref":"Protobuf.DSL.html#extend/4"},{"type":"macro","title":"Protobuf.DSL.extensions/1","doc":"Define extensions range in the message module to allow extensions for this module.\n\nExtension ranges are defined as a list of tuples `{start, end}`, where each tuple is\nan **exclusive** range starting and `start` and ending at `end` (the equivalent\nof `start..end-1` in Elixir).\n\nTo simulate the Protobuf `max` keyword, you can use `Protobuf.Extension.max/0`.","ref":"Protobuf.DSL.html#extensions/1"},{"type":"macro","title":"Examples - Protobuf.DSL.extensions/1","doc":"These Protobuf definition:\n\n```protobuf\nmessage Foo {\n  extensions 1, 10 to 20, 100 to max;\n}\n```\n\nWould be translated in Elixir to:\n\n    extensions [{1, 2}, {10, 21}, {100, Protobuf.Extension.max()}]","ref":"Protobuf.DSL.html#extensions/1-examples"},{"type":"macro","title":"Protobuf.DSL.field/3","doc":"Define a field for the message.\n\nCorresponds to Protobuf declarations such as:\n\n    string query = 1;\n\nor more generally\n\n        =  ;","ref":"Protobuf.DSL.html#field/3"},{"type":"macro","title":"Arguments - Protobuf.DSL.field/3","doc":"* `name` — must be an atom representing the name of the field.\n  * `field_number` — must be an integer representing the field number.\n  * `options` - a keyword list of options, see below.","ref":"Protobuf.DSL.html#field/3-arguments"},{"type":"macro","title":"Options - Protobuf.DSL.field/3","doc":"* `:proto3_optional` - boolean representing whether the field is optional with the `proto3`\n    syntax. See [the\n    documentation](https://developers.google.com/protocol-buffers/docs/proto3#oneof)\n\n  * `:type` - atom representing the type of the field.\n\n  * `:repeated` - boolean representing whether the field is repeated.\n\n  * `:optional` - boolean representing whether the field is optional.\n\n  * `:required` - boolean representing whether the field is required.\n\n  * `:enum` - boolean representing whether the field is a possible value of an enum.\n\n  * `:map` - boolean representing whether the field is a part of a map.\n\n  * `:default` - the default value of the field. Must be a valid Elixir term at compile time,\n    that can be encoded with Protobuf and matches with the type of the field.\n\n  * `:packed` - boolean representing whether the field is packed.\n\n  * `:deprecated` - boolean representing whether the field is deprecated.\n\n  * `:json_name` - if present, specifies the name of the field when using the JSON mapping (see\n    `Protobuf.JSON`). If not present, the default mapping will be used.","ref":"Protobuf.DSL.html#field/3-options"},{"type":"function","title":"Protobuf.DSL.on_def/6","doc":"","ref":"Protobuf.DSL.html#on_def/6"},{"type":"macro","title":"Protobuf.DSL.oneof/2","doc":"Define oneof in the message module.","ref":"Protobuf.DSL.html#oneof/2"},{"type":"module","title":"Protobuf.Extension","doc":"[Extensions](https://developers.google.com/protocol-buffers/docs/proto#extensions)\nlet you set extra fields for previously defined messages(even for messages in other packages)\nwithout changing the original message.\n\nTo load extensions you should call `Protobuf.load_extensions/0` when your application starts:\n\n    def start(_type, _args) do\n      Protobuf.load_extensions()\n      Supervisor.start_link([], strategy: :one_for_one)\n    end","ref":"Protobuf.Extension.html"},{"type":"module","title":"Examples - Protobuf.Extension","doc":"# protoc should be used to generate the code instead of writing by hand.\n    defmodule Foo do\n      use Protobuf, syntax: :proto2\n\n      extensions([{100, 101}, {1000, 536_870_912}])\n    end\n\n    # This module is generated for all \"extend\" calls in one file.\n    # This module is needed in `*_extension` function because the field name is scoped\n    # in the proto file.\n    defmodule Ext.PbExtension do\n      use Protobuf, syntax: :proto2\n\n      extend Foo, :my_custom, 1047, optional: true, type: :string\n    end\n\n    foo = %Foo{}\n    Foo.put_extension(foo, Ext.PbExtension, :my_custom, \"Custom field\")\n    Foo.get_extension(foo, Ext.PbExtension, :my_custom)","ref":"Protobuf.Extension.html#module-examples"},{"type":"function","title":"Protobuf.Extension.get/4","doc":"The actual function for `get_extension`","ref":"Protobuf.Extension.html#get/4"},{"type":"function","title":"Protobuf.Extension.max/0","doc":"Returns the maximum extension number.","ref":"Protobuf.Extension.html#max/0"},{"type":"function","title":"Examples - Protobuf.Extension.max/0","doc":"iex> Protobuf.Extension.max()\n    536870912","ref":"Protobuf.Extension.html#max/0-examples"},{"type":"function","title":"Protobuf.Extension.put/5","doc":"The actual function for `put_extension`","ref":"Protobuf.Extension.html#put/5"},{"type":"module","title":"Protobuf.JSON","doc":"JSON encoding and decoding utilities for Protobuf structs.\n\nIt follows Google's [specs](https://developers.google.com/protocol-buffers/docs/proto3#json)\nand reference implementation. Some features\nsuch as [well-known](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf)\ntypes are not fully supported yet.\n\nProto3 is supported as per the specification. Proto2 is supported in practice, but some of its\nfeatures might not work correctly, such as extensions.","ref":"Protobuf.JSON.html"},{"type":"module","title":"Types - Protobuf.JSON","doc":"| Protobuf                     | JSON                        | Supported |\n|------------------------------|-----------------------------|-----------|\n| `bool`                       | `true`/`false`              | Yes       |\n| `int32`, `fixed32`, `uint32` | Number                      | Yes       |\n| `int64`, `fixed64`, `uint64` | String                      | Yes       |\n| `float`, `double`            | Number                      | Yes       |\n| `bytes`                      | Base64 string               | Yes       |\n| `string`                     | String                      | Yes       |\n| `message`                    | Object (`{…}`)              | Yes       |\n| `enum`                       | String                      | Yes       |\n| `map `                   | Object (`{…}`)              | Yes       |\n| `repeated V`                 | Array of `[v, …]`           | Yes       |\n| `Any`                        | Object (`{…}`)              | Yes       |\n| `Timestamp`                  | RFC3339 datetime            | Yes       |\n| `Duration`                   | String (`seconds.fraction`) | Yes       |\n| `Struct`                     | Object (`{…}`)              | Yes       |\n| `Wrapper types`              | Various types               | Yes       |\n| `FieldMask`                  | String                      | Yes       |\n| `ListValue`                  | Array                       | Yes       |\n| `Value`                      | Any JSON value              | Yes       |\n| `NullValue`                  | `null`                      | Yes       |\n| `Empty`                      | Object (`{…}`)              | Yes       |","ref":"Protobuf.JSON.html#module-types"},{"type":"module","title":"Usage - Protobuf.JSON","doc":"`Protobuf.JSON` requires a JSON library to work, so first make sure you have `:jason` added\nto your dependencies:\n\n    defp deps do\n      [\n        {:jason, \"~> 1.2\"},\n        # ...\n      ]\n    end\n\nWith `encode/1` you can turn any `Protobuf` message struct into a JSON string:\n\n    iex> message = %Car{color: :RED, top_speed: 125.3}\n    iex> Protobuf.JSON.encode(message)\n    {:ok, \"{\\\"color\\\":\\\"RED\\\",\\\"topSpeed\\\":125.3}\"}\n\nAnd go the other way around with `decode/1`:\n\n    iex> json = ~S|{\"color\":\"RED\",\"topSpeed\":125.3}|\n    iex> Protobuf.JSON.decode(json, Car)\n    {:ok, %Car{color: :RED, top_speed: 125.3}}\n\nJSON keys are encoded as *camelCase strings* by default, specified by the `json_name` field\noption. So make sure to *recompile the `.proto` files in your project* before working with\nJSON encoding, the compiler will generate all the required `json_name` options. You can set\nyour own `json_name` for a particular field too:\n\n    message GeoCoordinate {\n      double latitude = 1 [ json_name = \"lat\" ];\n      double longitude = 2 [ json_name = \"long\" ];\n    }","ref":"Protobuf.JSON.html#module-usage"},{"type":"module","title":"Known Issues and Limitations - Protobuf.JSON","doc":"Currently, the `protoc` compiler won't check for field name collisions. This library won't\ncheck that either. Make sure your field names will be unique when serialized to JSON.\nFor instance, this message definition will not encode correctly since it will emit just\none of the two fields and the problem might go unnoticed:\n\n    message CollidingFields {\n      int32 f1 = 1 [json_name = \"sameName\"];\n      float f2 = 2 [json_name = \"sameName\"];\n    }\n\nAccording to the specification, when duplicated JSON keys are found in maps, the library\nshould raise a decoding error. It currently ignores duplicates and keeps the last occurrence.\n\n## `google.protobuf.Any`\n\nThe `google.protobuf.Any` type is supported. It can be used to encode and decode arbitrary\nmessages. When decoding, the \"type URL\" is used to determine the message type to decode\nto. The type URL is expected to be in the format of\n`type.googleapis.com/ . `. For example, the type URL for the\n`google.protobuf.Duration` message would be\n`type.googleapis.com/google.protobuf.Duration`. To determine the Elixir module from the\ntype URL, the package and message names are split on `.` and transformed into a module\nname. In the previous example, we'd end up with `Google.Protobuf.Duration`. Due to\narbitrary atom construction, we're forced to use `Module.safe_concat/1` to construct the\nmodule name. This means that the module must exist before decoding. If the module doesn't\nexist, decoding will raise an error.","ref":"Protobuf.JSON.html#module-known-issues-and-limitations"},{"type":"function","title":"Protobuf.JSON.decode/2","doc":"Decodes a JSON `iodata` into a `module` Protobuf struct.","ref":"Protobuf.JSON.html#decode/2"},{"type":"function","title":"Examples - Protobuf.JSON.decode/2","doc":"Given this Protobuf message:\n\n    syntax = \"proto3\";\n\n    message Car {\n      enum Color {\n        GREEN = 0;\n        RED = 1;\n      }\n\n      Color color = 1;\n      float top_speed = 2;\n    }\n\nYou can build its structs from JSON like this:\n\n    iex> Protobuf.JSON.decode(\"{}\", Car)\n    {:ok, %Car{color: :GREEN, top_speed: 0.0}}\n\n    iex> ~S|{\"color\":\"RED\"}| |> Protobuf.JSON.decode(Car)\n    {:ok, %Car{color: :RED, top_speed: 0.0}}\n\n    iex> ~S|{\"color\":\"GREEN\",\"topSpeed\":80.0}| |> Protobuf.JSON.decode(Car)\n    {:ok, %Car{color: :GREEN, top_speed: 80.0}}","ref":"Protobuf.JSON.html#decode/2-examples"},{"type":"function","title":"Protobuf.JSON.decode!/2","doc":"Decodes a JSON `iodata` into a `module` Protobuf struct.\n\nSimilar to `decode!/2` except it will unwrap the error tuple and raise in case of errors.","ref":"Protobuf.JSON.html#decode!/2"},{"type":"function","title":"Examples - Protobuf.JSON.decode!/2","doc":"iex> Protobuf.JSON.decode!(\"{}\", Car)\n    %Car{color: :GREEN, top_speed: 0.0}\n\n    iex> ~S|{\"color\":\"RED\"}| |> Protobuf.JSON.decode!(Car)\n    %Car{color: :RED, top_speed: 0.0}\n\n    iex> ~S|{\"color\":\"GREEN\",\"topSpeed\":80.0}| |> Protobuf.JSON.decode!(Car)\n    %Car{color: :GREEN, top_speed: 80.0}","ref":"Protobuf.JSON.html#decode!/2-examples"},{"type":"function","title":"Protobuf.JSON.encode/2","doc":"Generates a JSON representation of the given protobuf `struct`.","ref":"Protobuf.JSON.html#encode/2"},{"type":"function","title":"Options - Protobuf.JSON.encode/2","doc":"* `:use_proto_names` - use original field `name` instead of the camelCase `json_name` for\n    JSON keys. Defaults to `false`.\n  * `:use_enum_numbers` - encode `enum` field values as numbers instead of their labels.\n    Defaults to `false`.\n  * `:emit_unpopulated` - emit all fields, even when they are blank, empty, or set to their\n    default value. Defaults to `false`.","ref":"Protobuf.JSON.html#encode/2-options"},{"type":"function","title":"Examples - Protobuf.JSON.encode/2","doc":"Suppose that this is you Protobuf message:\n\n    syntax = \"proto3\";\n\n    message Car {\n      enum Color {\n        GREEN = 0;\n        RED = 1;\n      }\n\n      Color color = 1;\n      float top_speed = 2;\n    }\n\nEncoding is as simple as:\n\n    iex> Protobuf.JSON.encode(%Car{color: :RED, top_speed: 125.3})\n    {:ok, ~S|{\"color\":\"RED\",\"topSpeed\":125.3}|}\n\n    iex> Protobuf.JSON.encode(%Car{color: :GREEN})\n    {:ok, \"{}\"}\n\n    iex> Protobuf.JSON.encode(%Car{}, emit_unpopulated: true)\n    {:ok, ~S|{\"color\":\"GREEN\",\"topSpeed\":0.0}|}","ref":"Protobuf.JSON.html#encode/2-examples"},{"type":"function","title":"Protobuf.JSON.encode!/2","doc":"Generates a JSON representation of the given protobuf `struct`.\n\nSimilar to `encode/2` except it will unwrap the error tuple and raise in case of errors.","ref":"Protobuf.JSON.html#encode!/2"},{"type":"function","title":"Examples - Protobuf.JSON.encode!/2","doc":"iex> Protobuf.JSON.encode!(%Car{top_speed: 80.0})\n    ~S|{\"topSpeed\":80.0}|","ref":"Protobuf.JSON.html#encode!/2-examples"},{"type":"function","title":"Protobuf.JSON.encode_to_iodata/2","doc":"Similar to `encode/2`, but returns iodata\n\nThis function should be preferred to encode/2, if the generated JSON will be\nhanded over to one of the IO functions or sent over the socket. The Erlang\nruntime is able to leverage vectorised writes and avoid allocating a continuous\nbuffer for the whole resulting string, lowering memory use and increasing\nperformance.","ref":"Protobuf.JSON.html#encode_to_iodata/2"},{"type":"function","title":"Protobuf.JSON.encode_to_iodata!/2","doc":"Similar to `encode!/2`, but returns iodata\n\nSee `encode_to_iodata/2` for more information about when this function should\nbe preferred over `encode!/2`.","ref":"Protobuf.JSON.html#encode_to_iodata!/2"},{"type":"function","title":"Protobuf.JSON.from_decoded/2","doc":"Decodes a `json_data` map into a `module` Protobuf struct.\n\nSimilar to `decode/2` except it takes a JSON `map` representation of the data.\n  This is especially useful if you want to use custom JSON encoding or a custom\nJSON library.","ref":"Protobuf.JSON.html#from_decoded/2"},{"type":"function","title":"Examples - Protobuf.JSON.from_decoded/2","doc":"iex> Protobuf.JSON.from_decoded(%{}, Car)\n    {:ok, %Car{color: :GREEN, top_speed: 0.0}}\n\n    iex> Protobuf.JSON.from_decoded(%{\"color\" => \"RED\"}, Car)\n    {:ok, %Car{color: :RED, top_speed: 0.0}}\n\n    iex> Protobuf.JSON.from_decoded(%{\"color\" => \"GREEN\",\"topSpeed\" => 80.0}, Car)\n    {:ok, %Car{color: :GREEN, top_speed: 80.0}}","ref":"Protobuf.JSON.html#from_decoded/2-examples"},{"type":"function","title":"Protobuf.JSON.to_encodable/2","doc":"Generates a JSON-encodable map for the given Protobuf `struct`.\n\nSimilar to `encode/2` except it will return an intermediate `map` representation.\nThis is especially useful if you want to use custom JSON encoding or a custom\nJSON library.\n\nSupports the same options as `encode/2`.","ref":"Protobuf.JSON.html#to_encodable/2"},{"type":"function","title":"Examples - Protobuf.JSON.to_encodable/2","doc":"iex> Protobuf.JSON.to_encodable(%Car{color: :RED, top_speed: 125.3})\n    {:ok, %{\"color\" => :RED, \"topSpeed\" => 125.3}}\n\n    iex> Protobuf.JSON.to_encodable(%Car{color: :GREEN})\n    {:ok, %{}}\n\n    iex> Protobuf.JSON.to_encodable(%Car{}, emit_unpopulated: true)\n    {:ok, %{\"color\" => :GREEN, \"topSpeed\" => 0.0}}","ref":"Protobuf.JSON.html#to_encodable/2-examples"},{"type":"type","title":"Protobuf.JSON.encode_opt/0","doc":"","ref":"Protobuf.JSON.html#t:encode_opt/0"},{"type":"type","title":"Protobuf.JSON.json_data/0","doc":"","ref":"Protobuf.JSON.html#t:json_data/0"},{"type":"module","title":"Protobuf.Protoc.CLI","doc":"`protoc` plugin for generating Elixir code.\n\n`protoc-gen-elixir` (this name is important) **must** be in `$PATH`. You are not supposed\nto call it directly, but only through `protoc`.","ref":"Protobuf.Protoc.CLI.html"},{"type":"module","title":"Examples - Protobuf.Protoc.CLI","doc":"$ protoc --elixir_out=./lib your.proto\n    $ protoc --elixir_out=plugins=grpc:./lib/ *.proto\n    $ protoc -I protos --elixir_out=./lib protos/namespace/*.proto\n\nOptions:\n\n  * --version       Print version of protobuf-elixir\n  * --help (-h)     Print this help","ref":"Protobuf.Protoc.CLI.html#module-examples"},{"type":"function","title":"Protobuf.Protoc.CLI.supported_features/0","doc":"","ref":"Protobuf.Protoc.CLI.html#supported_features/0"},{"type":"behaviour","title":"Protobuf.TransformModule","doc":"Behaviour for transformer modules.\n\nBy defining a `transform_module/0` function on your protobuf message module\nyou can add custom encoding and decoding logic for your message.\n\nThe `c:Protobuf.new/1` function will not be called for structs that have a transform module, if\nyou still want to emulate this behavior you can use `Protobuf.TransformModule.InferFieldsFromEnum`.\n\nAs an example we can use this to implement a message that will be decoded as a string value:\n\n    defmodule StringMessage do\n      use Protobuf, syntax: :proto3\n\n      field :value, 1, type: :string\n\n      def transform_module(), do: MyTransformModule\n    end\n\nThe transformer behaviour implementation:\n\n    defmodule MyTransformModule do\n      @behaviour Protobuf.TransformModule\n\n      defmacro typespec(_default_ast) do\n        quote do\n          @type t() :: String.t()\n        end\n      end\n\n      @impl true\n      def encode(string, StringMessage) when is_binary(string), do: struct(StringMessage, value: string)\n\n      @impl true\n      def decode(%{value: string}, StringMessage), do: string\n    end\n\nNotice that since the `c:typespec/1` macro was introduced, transform modules can't\ndepend on the types that they transform anymore in compile time, meaning struct\nsyntax can't be used.","ref":"Protobuf.TransformModule.html"},{"type":"callback","title":"Protobuf.TransformModule.decode/2","doc":"Takes any protobuf message and the message type and encodes it into arbitrary\nElixir term.\n\nCalled after a message is decoded.","ref":"Protobuf.TransformModule.html#c:decode/2"},{"type":"callback","title":"Protobuf.TransformModule.encode/2","doc":"Takes any Elixir term and the protobuf message type and encodes it into\nthat type.\n\nCalled before a message is encoded.","ref":"Protobuf.TransformModule.html#c:encode/2"},{"type":"macrocallback","title":"Protobuf.TransformModule.typespec/1","doc":"Transforms the typespec for modules using this transformer.\n\nIf this callback is not present, the default typespec will be used.","ref":"Protobuf.TransformModule.html#c:typespec/1"},{"type":"type","title":"Protobuf.TransformModule.message/0","doc":"","ref":"Protobuf.TransformModule.html#t:message/0"},{"type":"type","title":"Protobuf.TransformModule.type/0","doc":"","ref":"Protobuf.TransformModule.html#t:type/0"},{"type":"type","title":"Protobuf.TransformModule.value/0","doc":"","ref":"Protobuf.TransformModule.html#t:value/0"},{"type":"module","title":"Protobuf.TransformModule.InferFieldsFromEnum","doc":"For structs with a `Protobuf.TransformModule` set, the `c:Protobuf.new/1` function\ndoes not get called. This transform module brings back the original behavior of calling\n`c:Protobuf.new/1` before the value is encoded.","ref":"Protobuf.TransformModule.InferFieldsFromEnum.html"},{"type":"exception","title":"Protobuf.DecodeError","doc":"An error for when decoding a Protobuf message fails.","ref":"Protobuf.DecodeError.html"},{"type":"exception","title":"Protobuf.EncodeError","doc":"An error for when encoding a Protobuf message fails.","ref":"Protobuf.EncodeError.html"},{"type":"exception","title":"Protobuf.ExtensionNotFound","doc":"","ref":"Protobuf.ExtensionNotFound.html"},{"type":"exception","title":"Protobuf.InvalidError","doc":"","ref":"Protobuf.InvalidError.html"},{"type":"exception","title":"Protobuf.JSON.DecodeError","doc":"","ref":"Protobuf.JSON.DecodeError.html"},{"type":"function","title":"Protobuf.JSON.DecodeError.new/1","doc":"","ref":"Protobuf.JSON.DecodeError.html#new/1"},{"type":"type","title":"Protobuf.JSON.DecodeError.t/0","doc":"","ref":"Protobuf.JSON.DecodeError.html#t:t/0"},{"type":"exception","title":"Protobuf.JSON.EncodeError","doc":"","ref":"Protobuf.JSON.EncodeError.html"},{"type":"function","title":"Protobuf.JSON.EncodeError.new/1","doc":"","ref":"Protobuf.JSON.EncodeError.html#new/1"},{"type":"type","title":"Protobuf.JSON.EncodeError.t/0","doc":"","ref":"Protobuf.JSON.EncodeError.html#t:t/0"},{"type":"extras","title":"protobuf-elixir","doc":"# protobuf-elixir\n\n[![CI](https://github.com/elixir-protobuf/protobuf/actions/workflows/main.yml/badge.svg)](https://github.com/elixir-protobuf/protobuf/actions/workflows/main.yml)\n[![Coverage Status](https://coveralls.io/repos/github/elixir-protobuf/protobuf/badge.svg?branch=main)](https://coveralls.io/github/elixir-protobuf/protobuf?branch=main)\n\nA pure Elixir implementation of [Google Protobuf](https://developers.google.com/protocol-buffers/).","ref":"readme.html"},{"type":"extras","title":"Why this instead of exprotobuf(gpb)? - protobuf-elixir","doc":"It has some must-have and other cool features like:\n\n1. A protoc [plugin](https://developers.google.com/protocol-buffers/docs/cpptutorial#compiling-your-protocol-buffers) to generate Elixir code just like what other official libs do, which is powerful and reliable.\n2. Generate **simple and explicit** code with the power of Macro. See [test/support/test_msg.ex](https://github.com/tony612/protobuf-elixir/blob/master/test/support/test_msg.ex).\n3. Plugins support. Only [grpc](https://github.com/tony612/grpc-elixir) is supported now.\n4. Use **structs** for messages instead of Erlang records.\n5. Support Typespec in generated code.","ref":"readme.html#why-this-instead-of-exprotobuf-gpb"},{"type":"extras","title":"Installation - protobuf-elixir","doc":"The package can be installed by adding `:protobuf` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:protobuf, \"~> 0.14.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Google Protos - protobuf-elixir","doc":"Since `:protobuf` version `0.14.0` we include all of the well known Google Protobuf modules. This conflicts with the deprecated `:google_protos` package. Please remove the `:google_protos` package from your dependencies and run `mix deps.unlock --unused`.","ref":"readme.html#google-protos"},{"type":"extras","title":"Features - protobuf-elixir","doc":"* Define messages with DSL\n* Decode basic messages\n* Skip unknown fields\n* Decode embedded messages\n* Decode packed and repeated fields\n* Encode messages\n* protoc plugin\n* map\n* Support default values\n* Validate values\n* Generate typespecs\n* oneof\n* (proto2) Extension (Experiment, see `Protobuf.Extension`)","ref":"readme.html#features"},{"type":"extras","title":"Usage - protobuf-elixir","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Generate Elixir code - protobuf-elixir","doc":"1. [Download](https://github.com/protocolbuffers/protobuf#protocol-compiler-installation) and\n   install the protocol buffer compiler (`protoc`). MacOS users can also install it through\n   Homebrew with `brew install protobuf`.\n\n2. Install `protoc` plugin `protoc-gen-elixir` for Elixir using the command below. Make sure the\n   `protoc-gen-elixir` binary is in your `PATH`. Either add `PATH=~/.mix/escripts:$PATH` to your\n   bash or zsh profile or, if you used asdf to install elixir, run `asdf reshim` and then verify\n   that `protoc-gen-elixir` works:\n\n    ```bash\n    $ mix escript.install hex protobuf 0.14.0\n    ```\n    Note: make sure to use the same version of the escript and dependency in your application.\n\n3. Generate Elixir code for [helloworld.proto](https://raw.githubusercontent.com/grpc/grpc/master/examples/protos/helloworld.proto) using `protoc`:\n\n    ```bash\n    $ protoc --elixir_out=./lib helloworld.proto\n    ```\n\n4. A `lib/helloworld.pb.ex` file will be generated, like:\n\n    ```elixir\n    defmodule Helloworld.HelloRequest do\n      @moduledoc false\n      use Protobuf, protoc_gen_elixir_version: \"0.10.0\", syntax: :proto3\n\n      field :name, 1, type: :string\n    end\n\n    defmodule Helloworld.HelloReply do\n      @moduledoc false\n      use Protobuf, protoc_gen_elixir_version: \"0.10.0\", syntax: :proto3\n\n      field :message, 1, type: :string\n    end\n    ```","ref":"readme.html#generate-elixir-code"},{"type":"extras","title":"Encode and decode in your code - protobuf-elixir","doc":"```elixir\nstruct = %Foo{a: 3.2, c: %Foo.Bar{}}\nencoded = Foo.encode(struct)\nstruct = Foo.decode(encoded)\n```\n\nValidation is done during encoding. An error will be raised if the struct is invalid: when it\nmisses a required field or has a mistyped value.","ref":"readme.html#encode-and-decode-in-your-code"},{"type":"extras","title":"Descriptor support - protobuf-elixir","doc":"If you use any custom options in your protobufs then to gain access to them you'll need to include\nthe raw descriptors in the generated modules. You can generate the descriptors by passing\n`gen_descriptors=true` in `--elixir_out`.\n\nThe descriptors will be available on each module from the `descriptor/0` function.\n\n```\n$ protoc --elixir_out=gen_descriptors=true:./lib/ *.proto\n$ protoc --elixir_out=gen_descriptors=true,plugins=grpc:./lib/ *.proto\n```","ref":"readme.html#descriptor-support"},{"type":"extras","title":"Package prefix - protobuf-elixir","doc":"You can use the `package_prefix` option to prefix generated Elixir code.\n\nFor example to prefix generated Elixir modules with `MyApp.Protos` use `my_app.protos` as package\nprefix:\n\n```\n$ protoc --elixir_out=./lib --elixir_opt=package_prefix=my_app.protos *.proto\n```","ref":"readme.html#package-prefix"},{"type":"extras","title":"Transformer module - protobuf-elixir","doc":"By defining a callback `transform_module/0` function on your protobuf message module\nyou can add custom encoding and decoding logic for your message. See the documentation\nfor `Protobuf.TransformModule` for more details.\n\nIf your protobufs are generated from a `.proto` files you can add the callback function\nby passing `transform_module=...` in `--elixir_out`.\n\n```\n$ protoc --elixir_out=transform_module=MyTransformModule:./lib/ *.proto\n```","ref":"readme.html#transformer-module"},{"type":"extras","title":"One file per module - protobuf-elixir","doc":"You can use the `one_file_per_module=true` option to change the way that files\nare generated into directories. By default, one `.pb.ex` file is generated for\neach `.proto` file you compile and each of those `.pb.ex` files can have\nmultiple Elixir module definitions in it.\n\nWith `one_file_per_module=true`, one `.pb.ex` file will be generated for each\ngenerated Elixir module and the directory structure will respect Elixir\nconventions. For example, a `MyPackage.MyMessage` message will end up in the\n`my_package/my_message.pb.ex` file.\n\n```\n$ protoc --elixir_out=one_file_per_module=true:./lib *.proto\n```","ref":"readme.html#one-file-per-module"},{"type":"extras","title":"Include documentation - protobuf-elixir","doc":"You can use the `include_docs=true` option to set the visibility of the\ngenerated modules documentation.\n\nWith `include_docs=true`, the generated modules will not have the\n`@module false` attribute.\n\n```\n$ protoc --elixir_out=./lib --elixir_opt=include_docs=true *.proto\n```","ref":"readme.html#include-documentation"},{"type":"extras","title":"gRPC Support - protobuf-elixir","doc":"If you write [services](https://developers.google.com/protocol-buffers/docs/proto#services) in\nprotobuf, you can generate [gRPC](https://github.com/elixir-grpc/grpc) code by passing\n`plugins=grpc` in `--elixir_out`:\n\n```\n$ protoc --elixir_out=plugins=grpc:./lib/ *.proto\n```","ref":"readme.html#grpc-support"},{"type":"extras","title":"Tips for protoc - protobuf-elixir","doc":"Custom protoc-gen-elixir name or path using `--plugin`:\n\n```bash\n$ protoc --elixir_out=./lib --plugin=./protoc-gen-elixir *.proto\n```\n\nPass `-I` argument if you import other protobuf files:\n\n```bash\n$ protoc -I protos --elixir_out=./lib protos/hello.proto\n```","ref":"readme.html#tips-for-protoc"},{"type":"extras","title":"Custom options - protobuf-elixir","doc":"Since extensions(`Protobuf.Extension`) is supported now, some options are defined, like custom\n`module_prefix`.\n\n1. Copy `src/elixirpb.proto` to your protos path.\n\n2. Import `elixirpb.proto` and use the options.\n\n    ```proto\n    syntax = \"proto2\";\n\n    package your.pkg;\n\n    import \"elixirpb.proto\";\n\n    option (elixirpb.file).module_prefix = \"Foo.Bar\";\n    ```\n\n3. Generate code as before.\n\nMore options will be added in the future, see `elixirpb.proto` comments for details.","ref":"readme.html#custom-options"},{"type":"extras","title":"Tests - protobuf-elixir","doc":"```bash\nmix test\n```","ref":"readme.html#tests"},{"type":"extras","title":"Sponsors - protobuf-elixir","doc":"* [Tubi](https://tubitv.com/)\n\n \n\n* [Community](https://www.community.com)","ref":"readme.html#sponsors"},{"type":"extras","title":"Acknowledgements - protobuf-elixir","doc":"Many thanks to [gpb](https://github.com/tomas-abrahamsson/gpb) and\n[golang/protobuf](https://github.com/golang/protobuf) as good examples of\nwriting Protobuf decoder/encoder.","ref":"readme.html#acknowledgements"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}